# Copyright 2024 Scaleway
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from qiskit.primitives import BackendEstimatorV2
from ..backends import AerBackend


class Estimator(BackendEstimatorV2):
    def __init__(
        self,
        backend: AerBackend,
        options: dict | None = None,
    ):
        if not isinstance(backend, AerBackend):
            raise Exception("backend must be instance of qiskit_scaleway.AerBackend")

        super().__init__(
            backend=backend,
            options=options,
        )
